/*----------------------------------------------------------------------------
 Copyright:      Hochsttter Bernhard
 Author:         Hochsttter Bernhard
 Remarks:        
 known Problems: 
 Version:        Juli 2010
 Description:    Dekoder fr Funk-Wetterstationen + Thermometer

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include <avr/io.h>
#include "usart.h"
#include "wx.h"

// Programmstart
int main(void)
{ 
    usart_init(BAUDRATE); 			// setup the UART
	usart_write("AVR-WX-Dekoder / DebLvl (0/1/2) = %i\n\r",dbg);

	// Schleife
	while(1)
	{
    	// Terminal -> Debug-Level Eingabe
		if ((UDR) != cmd)
		{  	
			if ((UDR) == 48){dbg = 0;}
       		if ((UDR) == 49){dbg = 1;}
			if ((UDR) == 50){dbg = 2;}
			usart_write("\n\rDebLvl = %i\n\r",dbg);
			cmd = (UDR);
		} // if UDR

		// Funksequenz aufrufen
		chk_funk();
	} // while
} // main

// Funkempfang testen *****************************************************
void chk_funk (void)
{
	// Variable initialisieren
	schleife = 0; rausch = 0; bits = 89; funkdat = 0;

	// Empfnger - Precheck -> 4 Pulse lesen
	do{sy[schleife] = 0;
		while ((PINA&0b00000001) == 0){sy[schleife]++;}schleife++;
		sy[schleife] = 0;
		while ((PINA&0b00000001) == 1){sy[schleife]--;}schleife++;
	}while (schleife < 4);

	// und auf Rauschsignal testen
	schleife = 1;	
	do{if ((abs(sy[schleife]) < 400)){rausch++;}
	schleife++;
	}while (schleife < 3);

	if (rausch == 0) 	// Precheck o.k. 
	{
		// Empfnger-Pulse in Array einlesen	
		do{sy[schleife] = 0;
			while ((PINA&0b00000001) == 0){sy[schleife]++;}schleife++;
			sy[schleife] = 0;
			while ((PINA&0b00000001) == 1){sy[schleife]--;}schleife++;
		}while (schleife < bits);

		chk_therm1(); 	// passende Wetterstationen prfen

	} // if rausch == 0
} // chk_funk

// Wetterstation testen *************************************************
// Hier: Standard Wetterstation 433 MHz / Temperatur / Luftfeuchte / Regenmessung

void chk_therm1 (void)
{
	rausch = 0;	funkdat = 0;

	// Startburst suchen ab Position 1 -> 0/1/0/1 lange Pulse
	schleife = 1;	
	do{
	if ((abs(sy[schleife]) < 400) || (abs(sy[schleife]) > 1600)){rausch++;}
	schleife++;
	}while (schleife < 4);
	
	if (rausch == 0) // Starburst ok -> weitermachen
	{
		// Start-Puls suchen -> kurzer Puls
		while ((abs(sy[schleife]) > 1000)  && (schleife < bits))
		{schleife++;} 

		// Start-Puls merken -> sy[0]
		sy[0]=(sy[schleife]); 			

		// Falls Start-Puls ok -> weitermachen	
		if ((sy[0] > 450) && (sy[0] < 700)) // 450 - 700 ist gut
		{
			sy[1] = schleife + 1; // Ab hier liegen Puls-Daten -> sy[1]

			// Endpuls (lang) suchen -> sy[2]
			do{schleife++;}while ((sy[schleife] > -3000) && (schleife < bits));
			sy[2] = schleife;

			if (sy[2] < 90)	// Endpuls im Array d.h. -> alle Datenpulse vorhanden
			{
				// Debug Pulse im Buffer
				if (dbg > 1)
				{
					usart_write("syn=%i dat=%i end=%i \n\r",sy[0],sy[1],sy[2]);
					schleife = 0;
					do{
					usart_write("Bit %i=%i ",schleife,sy[schleife]);schleife++;
					usart_write("%i=%i ",schleife,sy[schleife]);schleife++;
					usart_write("%i=%i ",schleife,sy[schleife]);schleife++;
					usart_write("%i=%i \n\r",schleife,sy[schleife]);schleife++;
					}while (schleife < bits);
					usart_write("\n\r");
				} // debug

  				// Funk-Temperaturdatendecoder, langer Impuls = 0  kurz = 1. 
				schleife = sy[1];	// Datenstart
				while (schleife <  sy[2])
				{
					if (sy[schleife] > 1000){datbit[funkdat++] = 0;} 
					if ((sy[schleife] > 400) && (sy[schleife] < 1000)){datbit[funkdat++] = 1;} 
					schleife++;
				} // while

				// Debug Bits im Buffer
				if (dbg > 0)
				{
					schleife =0;
					while (schleife < funkdat)
					{usart_write("%i",datbit[schleife]);schleife++;
					} // while
					usart_write("\r\n");
				} // dbg

				schleife = 0; 						// Komplettes Bitmuster bilden
				strcpy(uni_buff,""); 				// Stringbuffer leeren
				do{itoa (datbit[schleife],conv_buff,10);strcat(uni_buff,conv_buff);schleife++;}while (schleife < funkdat);
						
				sy[0] = 0;
				if (funkdat == 39) // Alle Daten da?
				{
 					if (strncmp(uni_buff,WXR,6)==0){sy[0] = 2;} // Regen
					if (strncmp(uni_buff,WXF,9)==0){sy[0] = 1;} // Feuchte
 
 					if (sy[0] == 0) // Temperatursensor
					{
						sy[1] = 0; schleife = 2; 	// Von 2 bis 10 Kennung ermitteln
						while (schleife < 11){sy[1] = sy[1] + datbit[schleife];schleife++;}
						strcpy(wx_k,"C S-");		// Kennung Standard-Sensor
						itoa (sy[1],conv_buff,10);strcat(wx_k,conv_buff);

						schleife = funkdat -24;		// ab hier Temp-Daten
						strcpy(wx_t,""); 			// Stringbuffer leeren

						sy[3] = ((8*datbit[schleife]+4*datbit[schleife+1]+2*datbit[schleife+2]+1*datbit[schleife+3])-5);
						sy[4] = ((8*datbit[schleife+4]+4*datbit[schleife+5]+2*datbit[schleife+6]+1*datbit[schleife+7]));
						sy[5] = ((8*datbit[schleife+8]+4*datbit[schleife+9]+2*datbit[schleife+10]+1*datbit[schleife+11]));

						// Minus-Grade -> Korrektur
						if (sy[3] < 0)
						{
							strcat(wx_t,"-");		// minus
							sy[3] = -1 -sy[3];		// Zehnerstelle	
							sy[4] =  9 -sy[4];		// Einerstelle
							sy[5] = 10 -sy[5];		// Nachkommastelle
							if (sy[4] ==  9){sy[4] = 0;sy[3] = sy[3]+1;} // bertrag
							if (sy[5] == 10){sy[5] = 0;sy[4] = sy[4]+1;} // bertrag
						} 

						itoa(sy[3],conv_buff,10);strcat(wx_t,conv_buff);
						itoa(sy[4],conv_buff,10);strcat(wx_t,conv_buff);strcat(wx_t,",");
						itoa(sy[5],conv_buff,10);strcat(wx_t,conv_buff);

						usart_write("%s %s %s\r\n",uni_buff,wx_t,wx_k);
 					} // if sy[0] = 0

					if (sy[0] == 1) // Feuchtesensor
					{
						schleife = funkdat -24;		// ab hier Daten
						strcpy(wx_f,""); 			// Stringbuffer leeren
						c = (8*datbit[schleife]+4*datbit[schleife+1]+2*datbit[schleife+2]+1*datbit[schleife+3]);
						itoa (c,conv_buff,10);strcat(wx_f,conv_buff);
						c = ((8*datbit[schleife+4]+4*datbit[schleife+5]+2*datbit[schleife+6]+1*datbit[schleife+7]));
						itoa (c,conv_buff,10);strcat(wx_f,conv_buff);

						usart_write("%s %s o/o\r\n",uni_buff,wx_f);
 					} // if sy[0] = 1

		   			if (sy[0] == 2) // Regensensor
					{
						schleife = funkdat -24;									// ab hier Daten
						strcpy(wx_r,"");										// Stringbuffer leeren
						c=0;nr=4096;
						while (nr >= 1) 										// 12 Bit Rechner
						{c = c+(nr*datbit[schleife]);nr = (nr/2);schleife++;}
						bits = c;												// merken
						if (c > 250){c = c - 10;} 								// Korrektur
	
						a_regen = (c/7); 										// Faktor Regensensor 7 ist gut
						itoa (a_regen,conv_buff,10);strcat(wx_r,conv_buff); 	// vom Sensor
	
						usart_write("%s %s mm (%i) \r\n",uni_buff,wx_r,bits);

 					} // if sy[0] = 2
				} // if funkdat = 39
			} // if ende < 90
		} // if Syncro ok
	} // if Starburst ok

	else{chk_therm2();} // weitere Funkempfnger testen

} // chk_therm 


// Weiteren Typ testen ********************************************************
// Hier: Medion Thermometer 433 MHz / Temperatur (Manchester Code)

void chk_therm2(void)
{
	rausch = 0;	funkdat = 0;
	
	// Startburst suchen ab Position 1 -> 0/1/0/1 
	schleife = 1;	
	do{
	if ((abs(sy[schleife]) < 1000) || (abs(sy[schleife]) > 5000)){rausch++;}
	schleife++;
	}while (schleife < 4);
	
	if (rausch == 0) // Starburst ok -> weitermachen
	{
		// Start-Puls suchen -> langer Puls
		while ((sy[schleife] > -5000)  && (schleife < bits))
		{schleife++;} 

		// Start-Puls merken -> sy[0]
		sy[0]=(sy[schleife]); 			

		// Falls Start-Puls ok -> weitermachen	
		if ((sy[0] > -8000) && (sy[0] < -5000)) // -8000 -5000 ist gut
		{
			sy[1] = schleife + 1; // Ab hier liegen Puls-Daten -> sy[1]

			// Endpuls (lang) suchen -> sy[2]
			do{schleife++;}while ((sy[schleife] > -8000) && (schleife < bits));
			sy[2] = schleife;

			if (sy[2] < 90)	// Endpuls im Array d.h. -> alle Datenpulse vorhanden
			{
				// Debug Pulse im Buffer
				if (dbg > 1)
				{
					usart_write("syn=%i dat=%i end=%i \n\r",sy[0],sy[1],sy[2]);
					schleife = 0;
					do{
					usart_write("Bit %i=%i ",schleife,sy[schleife]);schleife++;
					usart_write("%i=%i ",schleife,sy[schleife]);schleife++;
					usart_write("%i=%i ",schleife,sy[schleife]);schleife++;
					usart_write("%i=%i \n\r",schleife,sy[schleife]);schleife++;
					}while (schleife < bits);
					usart_write("\n\r");
				} // dbg

				// Kennung + Daten analysieren (Manchester Dekoder)
				schleife = sy[1];	// Datenstart
				sy[3] = 1; 			// Increment 1 oder 2 Bit

				while (schleife <  sy[2])
				{
					sy[3] = 0;
					// +kurz / -kurz --> 1
					if (((sy[schleife]>0)&&(sy[schleife]<2500)&&(sy[schleife+1]<0)&&(sy[schleife+1]> -2500))){datbit[funkdat++] = 1;sy[3]=2;} 
					// -kurz / +kurz --> 0
					if (((sy[schleife]<0)&&(sy[schleife]> -2500)&&(sy[schleife+1]>0)&&(sy[schleife+1]< 2500))){datbit[funkdat++] = 0;sy[3]=2;} 
					// +lang / -lang --> 1
					if (((sy[schleife]>0)&&(sy[schleife]>2500)&&(sy[schleife+1]<0)&&(sy[schleife+1]< -2500))){datbit[funkdat++] = 1;sy[3]=1;} 
					// -lang / +lang --> 0
					if (((sy[schleife]<0)&&(sy[schleife]< -2500)&&(sy[schleife+1]>0)&&(sy[schleife+1]> 2500))){datbit[funkdat++] = 0;sy[3]=1;} 
					// +lang / -kurz --> 1
					if (((sy[schleife]>0)&&(sy[schleife]>2500)&&(sy[schleife+1]<0)&&(sy[schleife+1]> -2500))){datbit[funkdat++] = 1;sy[3]=2;} 
					// -lang / +kurz --> 0
					if (((sy[schleife]<0)&&(sy[schleife]< -2500)&&(sy[schleife+1]>0)&&(sy[schleife+1]< 2500))){datbit[funkdat++] = 0;sy[3]=2;} 
					// +kurz / -lang --> 1
					if (((sy[schleife]>0)&&(sy[schleife]<2500)&&(sy[schleife+1]<0)&&(sy[schleife+1]< -2500))){datbit[funkdat++] = 1;sy[3]=1;} 
					// -kurz / +lang --> 0
					if (((sy[schleife]<0)&&(sy[schleife]> -2500)&&(sy[schleife+1]>0)&&(sy[schleife+1]> 2500))){datbit[funkdat++] = 0;sy[3]=1;} 
					schleife = schleife + sy[3];
					if (sy[3] == 0){funkdat=0;schleife = sy[2];} // Fehler
				} // while

				// Debug Bits im Buffer
				if (dbg > 0)
				{	
					schleife =0;
					while (schleife < funkdat)
					{usart_write("%i",datbit[schleife]);schleife++;
					} // while
					usart_write("\r\n");
				} // dbg

				if ((funkdat > 30)&&(funkdat < 33))		// Alle Daten da (31 bzw. 32)
				{
					schleife = 0; 						// Komplettes Bitmuster bilden
					strcpy(uni_buff,""); 				// Stringbuffer leeren
					do{itoa (datbit[schleife],conv_buff,10);strcat(uni_buff,conv_buff);schleife++;}while (schleife < funkdat);
											
					sy[1] = 0; schleife = 0; 			// Kennung ermitteln
					while (schleife < 5){sy[1] = sy[1] + datbit[schleife];schleife++;}
					strcpy(wx_k,"C M-");				// Kennung Medion-Sensor
					itoa (sy[1],conv_buff,10);strcat(wx_k,conv_buff);

					if (funkdat == 32){schleife = 0;}	// Datenstart einstellen
					if (funkdat == 31){schleife = -1;}	// Datenstart einstellen

					strcpy(wx_t,""); 					// Stringbuffer leeren

					sy[3] = ((8*datbit[schleife+19]+4*datbit[schleife+18]+2*datbit[schleife+17]+1*datbit[schleife+16]));
					sy[4] = ((8*datbit[schleife+15]+4*datbit[schleife+14]+2*datbit[schleife+13]+1*datbit[schleife+12]));
					sy[5] = ((8*datbit[schleife+11]+4*datbit[schleife+10]+2*datbit[schleife+9]+1*datbit[schleife+8]));

					// Minus Grade
					if (datbit[schleife+21] == 1){strcat(wx_t,"-");} 
					
					itoa(sy[3],conv_buff,10);strcat(wx_t,conv_buff);
					itoa(sy[4],conv_buff,10);strcat(wx_t,conv_buff);strcat(wx_t,",");
					itoa(sy[5],conv_buff,10);strcat(wx_t,conv_buff);

					usart_write("%s %s %s\r\n",uni_buff,wx_t,wx_k);
				} // if funkdat > 30
			} // if ende < 90
		} // if Syncro ok
	} // if Starburst ok

	else{chk_therm3();} // noch weitere Typen testen -> bei Bedarf erweiteren

} // void


// Weiteren Typ testen ********************************************************
// Platz fr weitere Dekodertypen

void chk_therm3(void)
{
	usart_write("Kein passender Dekoder\r\n");

} // void
